package org.example;

import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Semaphore;

public class Consumer implements Runnable {

    // Object for synchronization when incrementing numLines
    private Object synchronizationObject = new Object();

    // Queue of lines before processing them
    private Queue<String> linesQueue;

    // Queue of lines after processing them
    private ConcurrentLinkedQueue<String> processedLines;

    // Semaphore for controlling access to the  linesQueue
    private Semaphore linesQueueSemaphore;

    // Volatile variable to track the number of processed lines
    private volatile static long numLines = 0;

    // Getter for numLines
    public static long getNumLines() {
        return numLines;
    }

    // Constructor
    public Consumer(LinkedList<String> linesQueue, ConcurrentLinkedQueue<String> processedLines, Semaphore linesQueueSemaphore) {
        this.linesQueue = linesQueue;
        this.processedLines = processedLines;
        this.linesQueueSemaphore = linesQueueSemaphore;
    }

    // Run method to be executed when the thread starts
    @Override
    public void run() {
        try {
            // Infinite loop for continuous processing
            while (true) {
                try {
                    // Acquire the semaphore to process the lines
                    linesQueueSemaphore.acquire();
                } catch (InterruptedException i) {
                    // just for Handle InterruptedException
                }

                // Print the name of the current thread for debugging purposes
                System.out.println(Thread.currentThread().getName());

                // Increment the numLines variable in a synchronized block for thread safety
                synchronized (synchronizationObject) {
                    numLines++;
                }

                // Retrieve a line from the linesQueue
                String line = linesQueue.poll();

                if (line == null) {
                    break; // Exit the loop if no more lines
                }

                // Convert each lowercase letter to uppercase and vice versa
                StringBuilder swappedString = new StringBuilder();
                for (char c : line.toCharArray()) {
                    if (Character.isUpperCase(c)) {
                        swappedString.append(Character.toLowerCase(c));
                    } else if (Character.isLowerCase(c)) {
                        swappedString.append(Character.toUpperCase(c));
                    } else {
                        swappedString.append(c);
                    }
                }

                // Offer the processed line to the processedLines queue
                processedLines.offer(swappedString.toString());
            }
        } catch (Exception e) {
            // Print the exception stack trace for debugging purposes
            e.printStackTrace();
        }
    }
}
