package org.example;

import java.io.*;
import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.Semaphore;

public class Producer implements Runnable {

    // Static variable to keep track of the total number of lines produced
    private static long numLines = 0;

    // File to read lines from
    private String file;

    // Queue to store lines read from the file
    private Queue<String> linesQueue;

    // Semaphore to control access to the linesQueue
    private Semaphore linesQueueSemaphore;

    // Getter for numLines
    public static long getNumLines() {
        return numLines;
    }

    // Constructor
    public Producer(String file, LinkedList<String> linesQueue) {
        this.file = file;
        this.linesQueue = linesQueue;
        this.linesQueueSemaphore = new Semaphore(0);
    }


    // Getter for the linesQueueSemaphore
    public Semaphore getSemaphore() {
        return linesQueueSemaphore;
    }

    // Run method to be executed when the thread starts
    @Override
    public void run() {
        try (BufferedReader reader = new BufferedReader(new FileReader(file))) {
            String line;

            // Read lines from the file until the end is reached
            while ((line = reader.readLine()) != null) {
                // Offer the line to the linesQueue
                linesQueue.offer(line);

                // Increment the numLines variable
                numLines++;

                // Release the semaphore to signal that a new line is available for consumption
                linesQueueSemaphore.release();
            }
        } catch (IOException e) {
            // Print the exception stack trace for debugging purposes
            e.printStackTrace();
        }
    }
}
