package org.example;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;
import java.util.concurrent.*;

public class Writer {
    public static void main(String[] args) {
        // File paths
        String inputFile = "Input/input.txt";
        String outputFile = "Output/output.txt";

        // Determine the number of consumer threads
        int numConsumers = Runtime.getRuntime().availableProcessors();

        // Create a queue to hold the lines read from the file
        LinkedList<String> linesQueue = new LinkedList<>();

        // Create a concurrent queue to store the processed results for each consumer
        ConcurrentLinkedQueue<String> processedResults = new ConcurrentLinkedQueue<>();

        // Create a producer thread to read lines from the file and put them into the linesQueue
        Producer producer = new Producer(inputFile, linesQueue);
        Thread producerThread = new Thread(producer);
        producerThread.start();

        // Create a thread pool with the specified number of consumers
        ExecutorService executor = Executors.newFixedThreadPool(numConsumers);

        // Submit consumer tasks to the thread pool
        for (int i = 0; i < numConsumers; i++) {
            Consumer consumer = new Consumer(linesQueue, processedResults, producer.getSemaphore());
            executor.submit(consumer);
        }

        // Wait for the producer to finish
        try {
            producerThread.join();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }

        // Wait until all consumers finish processing the lines then shutdown the executor
        while (true) {
            if ( Producer.getNumLines() == Consumer.getNumLines()) {
                executor.shutdownNow();
                break;
            }
        }

        // Write the processed results to the output file
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile))) {
            for (String result : processedResults) {
                writer.write(result);
                writer.newLine();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
